from abc import ABC, abstractmethod
from typing import Optional

from cpl_query.extension import List

from bot_api.filter.auth_user_select_criteria import AuthUserSelectCriteria
from bot_data.filtered_result import FilteredResult
from bot_data.model.auth_user import AuthUser
from bot_data.model.auth_user_users_relation import AuthUserUsersRelation


class AuthUserRepositoryABC(ABC):

    @abstractmethod
    def __init__(self): pass

    @abstractmethod
    def get_all_auth_users(self) -> List[AuthUser]: pass

    @abstractmethod
    def get_filtered_auth_users(self, criteria: AuthUserSelectCriteria) -> FilteredResult: pass

    @abstractmethod
    def get_auth_user_by_email(self, email: str) -> AuthUser: pass

    @abstractmethod
    def find_auth_user_by_email(self, email: str) -> Optional[AuthUser]: pass

    @abstractmethod
    def find_auth_user_by_confirmation_id(self, id: str) -> Optional[AuthUser]: pass

    @abstractmethod
    def find_auth_user_by_forgot_password_id(self, id: str) -> Optional[AuthUser]: pass

    @abstractmethod
    def add_auth_user(self, user: AuthUser): pass

    @abstractmethod
    def update_auth_user(self, user: AuthUser): pass

    @abstractmethod
    def delete_auth_user(self, user: AuthUser): pass

    @abstractmethod
    def add_auth_user_user_rel(self, rel: AuthUserUsersRelation): pass

    @abstractmethod
    def update_auth_user_user_rel(self, rel: AuthUserUsersRelation): pass

    @abstractmethod
    def delete_auth_user_user_rel(self, rel: AuthUserUsersRelation): pass
